<?php

namespace Drp\SimpleCms\Generators\Commands;

use Drp\SimpleCms\Generators\MigrationCreator;
use Drp\SimpleCms\Generators\ViewCreator;
use Illuminate\Database\Console\Migrations\BaseCommand;
use Illuminate\Support\Composer;
use Symfony\Component\Console\Input\InputArgument;

/**
 * Class ContentMakeCommand
 * @package Drp\SimpleCms
 */
class ContentMakeCommand extends BaseCommand
{
    /**
     * The console command name.
     *
     * @var string
     */
    protected $name = 'make:content';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Create a new content class and apply schema at the same time';

    /**
     * @var MigrationCreator
     */
    private $migrationCreator;

    /**
     * @var Composer
     */
    private $composer;
    /**
     * @var ViewCreator
     */
    private $viewCreator;

    /**
     * ContentMakeCommand constructor.
     *
     * @param MigrationCreator $migrationCreator
     * @param ViewCreator $viewCreator
     * @param Composer $composer
     */
    public function __construct(
        MigrationCreator $migrationCreator,
        ViewCreator $viewCreator,
        Composer $composer
    ) {
        $this->migrationCreator = $migrationCreator;
        $this->composer = $composer;
        $this->viewCreator = $viewCreator;
        parent::__construct();
    }


    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function fire()
    {
        $this->name = $this->argument('name');

        $this->makeMigration($this->getMigrationName());

        $this->call('make:content-model', [
            'name' => $this->name
        ]);

        $this->viewCreator->create($this->getViewPath());
    }

    /**
     * Make the models migrations
     * @return void
     */
    private function makeMigration()
    {
        $file = pathinfo($this->migrationCreator->create(
            $this->getMigrationName(),
            $this->getMigrationPath(),
            $this->getTableName()
        ), PATHINFO_FILENAME);

        $this->line("<info>Created Migration:</info> {$file}");

        $this->composer->dumpAutoloads();
    }

    /**
     * Get the class name.
     *
     * @return string
     */
    protected function getMigrationName()
    {
        return 'create_' . strtolower(str_plural(snake_case($this->name))) . '_table';
    }

    /**
     * @return string
     */
    protected function getTableName()
    {
        return str_plural(snake_case($this->name));
    }

    /**
     * Get the console command arguments.
     *
     * @return array
     */
    protected function getArguments()
    {
        return [
            ['name', InputArgument::REQUIRED, 'The name of the content'],
        ];
    }

    /**
     * Get the path to the migration directory.
     *
     * @return string
     */
    protected function getMigrationPath()
    {
        return $this->laravel->databasePath() . DIRECTORY_SEPARATOR . 'migrations';
    }

    /**
     * Get the path to the migration directory.
     *
     * @return string
     */
    protected function getViewPath()
    {
        return $this->laravel->resourcePath() . DIRECTORY_SEPARATOR . implode(DIRECTORY_SEPARATOR, [
                'views', 'admin', 'content', lcfirst(studly_case(($this->name)))
            ]);
    }
}
